// --------------------------------------------------------------------
// -- Stochastic Simulation --
// -- Cornell MEng Project --
// -- By Tian Gao --
// -- Fall 2013 --
// -- Advisor: Bruce Land --
// -- on board DE2i-150 --
// --------------------------------------------------------------------

module DE2_TOP
	(
		////////////////////	Clock Input	 	////////////////////	 
		CLOCK_50,						//	50 MHz
		//////////////////// Push Button  //////////////////////
		KEY,
		////////////////////  SWITCH   ////////////////////////
		SW,
		////////////////////   HEX   ////////////////////////////
		HEX0,
		HEX1,
		HEX2,
		HEX3,
		HEX4,
		HEX5,
		HEX6,
		HEX7,
		///////////////////////  SSRAM   /////////////////////////
		SRAM_BE,
		SRAM0_CS_N,
		SRAM1_CS_N,
		SRAM_ADSC_N,
		SRAM_GW_N,
		SRAM_ADSP_N,
		SRAM_ADV_N,
		SRAM_CLK,
		SRAM_OE_N,
		SRAM_WE_N,
		SRAM_DQ,
		SRAM_ADDR,
		///////////////////	VGA		////////////////////////////
		VGA_CLK,   						//	VGA Clock
		VGA_HS,							//	VGA H_SYNC
		VGA_VS,							//	VGA V_SYNC
		VGA_BLANK,						//	VGA BLANK
		VGA_SYNC,						//	VGA SYNC
		VGA_R,   						//	VGA Red[9:0]
		VGA_G,	 						//	VGA Green[9:0]
		VGA_B  						//	VGA Blue[9:0]
	);

	////////////////////////	Clock Input	 	////////////////////////
	input			CLOCK_50;				//	50 MHz
	////////////////////////// Push Button  //////////////////////////
	input    [3:0]KEY;
	////////////////////////  SWITCH   /////////////////////////////
	input    [17:0]SW;
	////////////////////////   HEX      ///////////////////////////////
	output   [6:0] HEX0;
	output   [6:0] HEX1;
	output   [6:0] HEX2;
	output   [6:0] HEX3;
	output   [6:0] HEX4;
	output   [6:0] HEX5;
	output   [6:0] HEX6;
	output   [6:0] HEX7;
	////////////////////////   SRAM    ///////////////////////////////
	output 	[3:0]	SRAM_BE;
	output			SRAM0_CS_N;
	output			SRAM1_CS_N;
	output			SRAM_ADSC_N;
	output			SRAM_GW_N;
	output			SRAM_ADSP_N;
	output			SRAM_ADV_N;
	output			SRAM_CLK;
	output			SRAM_OE_N;
	output			SRAM_WE_N;
	inout   [31:0] SRAM_DQ;
	output  [19:0]	SRAM_ADDR;
	////////////////////////	VGA			////////////////////////////
	output			VGA_CLK;   				//	VGA Clock
	output			VGA_HS;					//	VGA H_SYNC
	output			VGA_VS;					//	VGA V_SYNC
	output			VGA_BLANK;				//	VGA BLANK
	output			VGA_SYNC;				//	VGA SYNC
	output	[7:0]	VGA_R;   				//	VGA Red[9:0]
	output	[7:0]	VGA_G;	 				//	VGA Green[9:0]
	output	[7:0]	VGA_B;   				//	VGA Blue[9:0]

	//////////////////////////////////////////////////////////////////////
	// CLOCK
	wire 		CLK_MAIN;

	// SRAM
	assign	SRAM_BE     =  4'b0000; // Byte enable
	assign	SRAM0_CS_N  =  1'b0;		// chip 0 enable
	assign	SRAM1_CS_N  =  1'b0;    // chip 1 disable
	assign	SRAM_ADSC_N =  1'b0;    // SRAM Controller Address Status
	assign   SRAM_GW_N   =  1'b1;    // SRAM global Write Enable
	//assign	SRAM_ADSP_N =  1'b1; 	// SSRAM Processor Address Status;
	assign	SRAM_ADV_N  =  1'b1;    // burst address advance
	assign	SRAM_OE_N   =  1'b0;    // SRAM output Enable
	//assign   SRAM_ADDR   =  19'b0;
	assign 	SRAM_CLK    =  CLOCK_50;
	//assign	SRAM_DQ		=	16'hzzzz;

	wire [31:0]	mSEG7_DIG;
	reg  [31:0]	Cont;
	wire		VGA_CTRL_CLK;
	wire		AUD_CTRL_CLK;
	wire [7:0]	mVGA_R;
	wire [7:0]	mVGA_G;
	wire [7:0]	mVGA_B;
	wire [19:0]	mVGA_ADDR;			//video memory address
	wire [9:0]  Coord_X, Coord_Y;	//display coods
	wire		DLY_RST;

	assign	TD_RESET	=	1'b1;	//	Allow 27 MHz input
	//assign	AUD_ADCLRCK	=	AUD_DACLRCK;
	//assign	AUD_XCK		=	AUD_CTRL_CLK;


	Reset_Delay			r0	(	.iCLK(CLOCK_50),.oRESET(DLY_RST)	);

	VGA_Audio_PLL 		p1	(	.areset(~DLY_RST),.inclk0(CLOCK_50),.c0(VGA_CTRL_CLK),.c2(VGA_CLK)	);

	MAIN_PLL				p2 ( .areset(~DLY_RST),.inclk0(CLOCK_50),.c0(CLK_MAIN) );
	VGA_Controller		u1	(	//	Host Side
								.iCursor_RGB_EN(4'b0111),
								.oAddress(mVGA_ADDR),
								.oCoord_X(Coord_X),
								.oCoord_Y(Coord_Y),
								.iRed(mVGA_R),
								.iGreen(mVGA_G),
								.iBlue(mVGA_B),
								//	VGA Side
								.oVGA_R(VGA_R),
								.oVGA_G(VGA_G),
								.oVGA_B(VGA_B),
								.oVGA_H_SYNC(VGA_HS),
								.oVGA_V_SYNC(VGA_VS),
								.oVGA_SYNC(VGA_SYNC),
								.oVGA_BLANK(VGA_BLANK),
								//	Control Signal
								.iCLK(VGA_CTRL_CLK),
								.iRST_N(DLY_RST)	);



	assign  mVGA_R = {Coord_X[0]? SRAM_DQ[0]:SRAM_DQ[15], 7'b0} ;
	assign  mVGA_G = {Coord_X[0]? SRAM_DQ[1]:SRAM_DQ[14], 7'b0};
	assign  mVGA_B = {Coord_X[0]? SRAM_DQ[2]:SRAM_DQ[13], 7'b0} ;
	//assign  mVGA_R = {SRAM_DQ[15:11],3'b0} ;
	//assign  mVGA_G = {SRAM_DQ[10:6],3'b0};
	//assign  mVGA_B = {SRAM_DQ[5:0],2'b0} ;
	//SIR MODULE
	wire [8:0]sumI;
	wire SIR_CLK,SIR_RST,SIR_RESET_I;
	wire ADDER_CLK;
	wire [150:1]SIR_S;
	wire [150:1]SIR_I;

	//VGA stuff

	wire IsReadFlag;					//whether finish writing, 0 for not read, 1 for read
	reg Start;
	wire [9:0]WriteAddrX;				//SRAM X address for writing
	wire [8:0]WriteAddrY;				//SRAM Y address for writing
	wire [15:0]WriteInBits;			//16bit temporary write in SRAM
	wire VGA_CAL_CLK;
	//wire [7:0]counter_fix = 8'b00100000;	//VGA prescalar
	wire [7:0]VGA_counter_fix = {SW[7:1],1'b1};


	//Here is my code
	//assign SIR_CLK = SW[10];
	assign SIR_CLK = CLK_MAIN;
	assign ADDER_CLK = CLK_MAIN;
	assign VGA_CAL_CLK = CLK_MAIN;
	wire [15:0]temp_out;
	
/*	HexDigit HD0 
	(
		.segs	(HEX0),
		.num	(temp_out[3:0])
	);
	HexDigit HD1 
	(
		.segs	(HEX1),
		.num	(temp_out[7:4])
	);
	HexDigit HD2 
	(
		.segs	(HEX2),
		.num	(temp_out[11:8])
	);
	HexDigit HD3 
	(
		.segs	(HEX3),
		.num	(temp_out[15:12])
	);*/

	RST_DELAY RST0		//delay the reset signal to avoid glitch
	(
		.clk(VGA_CAL_CLK),
		.rst_in(~KEY[0]),
		.rst_out(SIR_RST)
	);
	RST_DELAY RST1
	(
		.clk(VGA_CAL_CLK),
		.rst_in(~KEY[1]),
		.rst_out(SIR_RESET_I)
	);
	Infected_Adder_Tree ADDR1(ADDER_CLK,SIR_RST,SIR_I,sumI);
	SIR SIR1(
				.clk(SIR_CLK),
				.Reset_All(SIR_RST),
				.Reset_I(SIR_RESET_I),
				.S(SIR_S),
				.I(SIR_I),
				//.temp_out(temp_out)
			  );
	VGA_DISPLAY VGA_DIS(
								.VGA_CAL_CLK(VGA_CAL_CLK),
								.VGA_RST(SIR_RST | SIR_RESET_I),
								.Coord_X(Coord_X),
								.Coord_Y(Coord_Y),
								.WriteAddrX(WriteAddrX),
								.WriteAddrY(WriteAddrY),
								.WriteInBits(WriteInBits),
								.sumI(sumI),
								.IsReadFlag(IsReadFlag),
								.counter_fix(VGA_counter_fix)
							);

	assign SRAM_WE_N     = IsReadFlag? 1'b1 : 1'b0;
	assign SRAM_DQ[15:0] = IsReadFlag? 16'hzzzz : 			//if reading, let read
							                 WriteInBits;		//if writing, write 16 bits 
	assign SRAM_ADDR     = IsReadFlag ? 
							                 {Coord_X[9:1],Coord_Y[8:0]}: 		// if it's reading, get 6bits X and 9bits Y because memory has 16bits
							                 {WriteAddrX[9:1], WriteAddrY};
	assign SRAM_ADSP_N   = IsReadFlag ? 1'b0 : 1'b1;
	

endmodule //top module

module RST_DELAY 
(
	clk,
	rst_in,
	rst_out
);
	input clk;
	input rst_in;
	output reg rst_out;
	reg [19:0] counter;
	always @ (posedge clk)
	begin
		if (rst_in == 1'b0)
		begin
			counter <= 20'b0;
			rst_out <= 1'b0;
		end
		else
		begin
			counter = counter + 1'b1;
			if (counter == {1'b1,19'b0})
			begin
				rst_out <= 1'b1;
			end
		end
	end
endmodule

